use std::ops::Range;
use std::time::Instant;

pub fn execute_task01(content: &str) {
    let start = Instant::now();
    let product_correct_races = solve_01(content).unwrap();
    let duration = start.elapsed();

    assert_eq!(1624896, product_correct_races);
    println!("Day06 - Task01 - Duration: {duration:?} - Product Correct Races: {}", product_correct_races)
}

pub fn execute_task02(content: &str) {
    let start = Instant::now();
    let product_correct_races = solve_02(content).unwrap();
    let duration = start.elapsed();

    assert_eq!(32583852, product_correct_races);
    println!("Day06 - Task02 - Duration: {duration:?} - Correct Race: {}", product_correct_races)
}

fn solve_01(content: &str) -> anyhow::Result<i64> {
    let result = content
        .lines()
        .map(|line| line
            .split_once(":")
            .unwrap()
            .1
            .trim())
        .map(|line| line
            .split(" ")
            .filter(|part| part.len() > 0)
            .map(|data| data.parse::<i64>().unwrap())
            .collect::<Vec<i64>>())
        .collect::<Vec<Vec<i64>>>();

    let mut races = vec![];

    for i in 0..result[0].len() {
        races.push((result[0][i].clone(), result[1][i].clone()))
    }

    Ok(races
        .iter()
        .map(|(time, distance)| calc_possible_options(*time, *distance))
        .map(|options| options.end - options.start)
        .product())
}

fn solve_02(content: &str) -> anyhow::Result<i64> {
    let result = content
        .lines()
        .map(|line| line
            .split_once(":")
            .unwrap()
            .1
            .trim())
        .map(|line| line
            .replace(" ", "")
            .parse::<i64>().unwrap())
        .collect::<Vec<i64>>();

    let races = vec![(result[0].clone(), result[1].clone())];
    Ok(races
        .iter()
        .map(|(time, distance)| calc_possible_options(*time, *distance))
        .map(|options| options.end - options.start)
        .product())
}


fn calc_possible_options(time: i64, distance: i64) -> Range<i64> {
    // f(x) = x * (time - x)
    // f(x) = time * x - x * x
    // PQ Ready: 0 = x * x - time * x + distance
    // =>
    // p = - time
    // q = distance

    let time = time as f64;
    let distance = distance as f64;

    let x1 = time / 2.0 - ((time * time / 4.0) - distance).sqrt();
    let x2 = time / 2.0 + ((time * time / 4.0) - distance).sqrt();

    (x1.floor() as i64 + 1)..(x2.ceil() as i64)
}

#[test]
fn test_solve_01() {
    let example = r#"Time:      7  15   30
Distance:  9  40  200"#;

    let result = solve_01(example).unwrap();

    assert_eq!(288, result);
}

#[test]
fn test_solve_02() {
    let example = r#"Time:      7  15   30
Distance:  9  40  200"#;

    let result = solve_02(example).unwrap();

    assert_eq!(71503, result);
}
